//=============================================================================
// Display_OLED.cpp - implementation for the OLED version of display for the
//            remote.
//=============================================================================
#include "globals.h"
#include "OLEDSGC.h"



//=============================================================================
// Define some information like Buttons and the like that we are using.
//=============================================================================
#define KPD_BWIDTH     38 // kludge for X2 should be X1+width-1... 
#define KPD_BHEIGHT    36     
#define KPD_DELTAX     40
#define KPD_ROW1_Y     152
#define KPD_ROW2_Y     195
#define DCLR_BUTTON_GREY   0xc718
#define DCLR_BUTTON_GREEN    ILI9341_GREEN
#define DCLR_BUTTON_YELLOW   ILI9341_YELLOW

#define NUM_BUTTONS 19
BUTTON g_abtns[NUM_BUTTONS];        // define our set of buttons...
char *g_apszBtns[NUM_BUTTONS] = {
  "0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F","Up","Dn","Ok"};

SLIDER g_asldrs[2];    // try 2 sliders...

//=============================================================================
// InitOLED: Initialize our OLED display
//=============================================================================
void RemoteDisplay::Init(void) {                // lets initialize our display
  word wX;

  g_plcd.Init(-1);
  // Now Initialize all of our buttons...
  wX = 1;
  // First the hex buttons
  for (byte i = 0; i < 8; i++) {
    g_abtns[i].Init(wX, KPD_ROW1_Y,  KPD_BWIDTH, KPD_BHEIGHT, DCLR_BUTTON_GREY, DCLR_BUTTON_GREEN, DCLR_BLACK, g_apszBtns[i], 0x100+i);
    g_abtns[i+8].Init(wX, KPD_ROW2_Y,  KPD_BWIDTH, KPD_BHEIGHT, DCLR_BUTTON_GREY, DCLR_BUTTON_GREEN, DCLR_BLACK, g_apszBtns[i+8], 0x100+i+8);
    wX += KPD_DELTAX;
  }

  // Then the others.
  g_abtns[16].Init(270, 10, 48, 34, DCLR_BUTTON_GREY, DCLR_BUTTON_YELLOW, DCLR_BLACK, g_apszBtns[16], 0x204);
  g_abtns[17].Init(270, 55, 48, 34, DCLR_BUTTON_GREY, DCLR_BUTTON_YELLOW, DCLR_BLACK, g_apszBtns[17], 0x208);
  g_abtns[18].Init(270, 100, 48, 34, DCLR_BUTTON_GREY, DCLR_BUTTON_YELLOW, DCLR_BLACK, g_apszBtns[18], 0x202);

  void Init (word x, word y, word dx, word dy, word wClr, word wHiClr, word wBackClr, word wMin, word wMax, word wVal);
  // Try some sliders...
  g_asldrs[0].Init(208, 10, 20, 120,  DCLR_BUTTON_YELLOW, DCLR_GREEN, DCLR_BLACK, 0, 255, 127);
  g_asldrs[1].Init(240, 10, 20, 120,  DCLR_BUTTON_YELLOW, DCLR_RED, DCLR_BLACK, 0, 255, 127);
}  


//=============================================================================
//=============================================================================
void RemoteDisplay::SwitchToDisplayMode(byte bMode) {
  byte i;

  g_plcd.ClearScreen();

  // Lets start off with all of the buttons displayed on all of the pages...
  // We are table driving the buttons...
  for (i=0; i < NUM_BUTTONS; i++) {
    g_abtns[i].SetPressed(false);
    g_abtns[i].Draw();
  }

  // Lets try some simple text output for now.
  switch (bMode) {
  case MODE_NORMAL:
  default:
    g_plcd.DisplayString(0, 1, 2, DCLR_WHITE, "Left");
    g_plcd.DisplayString(13, 1, 2, DCLR_WHITE, "Right");


    g_asldrs[0].Draw();
    g_asldrs[1].Draw();

    g_plcd.SetFontBackgroundColor(DCLR_WHITE);

    // make sure we display each of the values.
    _awJoyX[0] = 0xffff;
    _awJoyX[1] = 0xffff;
    _awJoyY[0] = 0xffff;
    _awJoyY[1] = 0xffff;
    _awAccelX[0] = 0xffff;
    _awAccelX[1] = 0xffff;
    _awAccelY[0] = 0xffff;
    _awAccelY[1] = 0xffff;
    _awAccelZ[0] = 0xffff;
    _awAccelZ[1] = 0xffff;
    _wRSlider = 0xffff;                // Likewise for sliders
    _wLSlider = 0xffff;
    _wMSlider = 0xffff;
    _abBatPercent[0] = 0xff;
    _abBatPercent[1] = 0xff;
    _abBatPercent[2] = 0xff;
    _fConnected = 0;
    break;

  case MODE_CHANGE_DEST_LIST:
    g_plcd.DisplayString(0, 0, 2, DCLR_WHITE, "Choose Dest");
    g_plcd.DisplayString(0, 1, 2, DCLR_GREEN, "RJoy Select Item");
    g_plcd.DisplayString(0, 2, 2, DCLR_GREEN, "LJoy Btn - Scan");
    g_plcd.DisplayString(0, 3, 2, DCLR_GREEN, "Keypad Direct");
    g_plcd.DisplayString(0, 7, 2, DCLR_WHITE, "Current:");
    g_plcd.DisplayString(0, 8, 2,DCLR_WHITE, "New:");  


    break;

  case MODE_CHANGE_MY:
    g_plcd.DisplayString(0, 0, 2, DCLR_WHITE, "Choose My");
    g_plcd.DisplayString(0, 7, 2, DCLR_WHITE, "Current:");
    g_plcd.DisplayString(0, 8, 2,DCLR_WHITE, "New:");  
    break;

  case MODE_CALIBRATE:
    g_plcd.DisplayString(0, 0, 2, DCLR_WHITE, "Calibrate");
    g_plcd.DisplayString(0, 1, 2, DCLR_GREEN, "Move All Joysticks");
    g_plcd.DisplayString(0, 2, 2, DCLR_GREEN, "and sliders to");
    g_plcd.DisplayString(0, 3, 2, DCLR_GREEN, "to full extent");
    break;    
  }
}


//=============================================================================
//=============================================================================
void RemoteDisplay::StartDispalyUpdate(void) {
  //    OLEDSerial.flush();    // simply flush anything that may have been stuck here.
}

//=============================================================================
// DisplayJoytick : display information about one joystick.
//=============================================================================
void RemoteDisplay::DisplayJoystick(byte iJoy, byte x, byte y) {
  byte bCol;
  byte bColP;
  byte bJoyVal;

  bCol = iJoy? 13 : 0;
  bColP = bCol * 8 + 28;        // convert to pixel offsets	

  g_plcd.SetSolidPin(true);
  g_plcd.SetFontBackgroundColor(-1);

  // First lets output the Text values

  if (x != _awJoyX[iJoy]) {
    _awJoyX[iJoy] = x;
    g_plcd.DisplayNum(bCol, 2, 2, DCLR_RED, x, 3);        // X - Red

    //X StickUpdate
    bJoyVal = x/4;     // Range of 0-63...
    g_plcd.DrawRect(bColP, 25, bColP + bJoyVal, 34, DCLR_WHITE);          // White square
    g_plcd.DrawRect(bColP + bJoyVal, 25, bColP + 64, 34, DCLR_BLACK);     // black Square
  }


  if (y != _awJoyY[iJoy]) {
    _awJoyY[iJoy] = y;
    g_plcd.DisplayNum(bCol, 3, 2, DCLR_RED, y, 3);        // Y

    bJoyVal = y/4;     // Range of 0-63...
    g_plcd.DrawRect(bColP, 37, bColP + bJoyVal, 46, DCLR_GREEN);          // Green square
    g_plcd.DrawRect(bColP + bJoyVal, 37, bColP + 64, 46, DCLR_BLACK);     // black Square
  }
}    

//=============================================================================
// DisplayJoytick : display information about one joystick.
//=============================================================================
void RemoteDisplay::DisplayAccels(byte iAccel, byte x, byte y, byte z) {
  byte bCol;
  byte bColP;
  byte bAccelVal;

  bCol = iAccel? 13 : 0;
  bColP = bCol * 8 + 28;        // convert to pixel offsets	

#define ACCELTEXTY 4
#define ACCELGRAPHICY(irow)   ((irow)*12)
  g_plcd.SetSolidPin(true);

  // First lets output the Text values
  if (x != _awAccelX[iAccel]) {
    _awAccelX[iAccel] = x;
    g_plcd.DisplayNum(bCol, ACCELTEXTY, 2, DCLR_RED, x, 3);        // X - Red

    //X StickUpdate
    bAccelVal = x/4;     // Range of 0-63...
    g_plcd.DrawRect(bColP, ACCELGRAPHICY(ACCELTEXTY)+1, bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY)+10, DCLR_WHITE);          // White square
    g_plcd.DrawRect(bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY)+1, bColP + 64, ACCELGRAPHICY(ACCELTEXTY)+10, DCLR_BLACK);     // black Square
  }


  if (y != _awAccelY[iAccel]) {
    _awAccelY[iAccel] = y;
    g_plcd.DisplayNum(bCol, ACCELTEXTY+1, 2, DCLR_RED, y, 3);        // Y

    bAccelVal = y/4;     // Range of 0-63...
    g_plcd.DrawRect(bColP, ACCELGRAPHICY(ACCELTEXTY+1)+1, bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY+1)+10, DCLR_GREEN);          // Green square
    g_plcd.DrawRect(bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY+1)+1, bColP + 64, ACCELGRAPHICY(ACCELTEXTY+1)+10, DCLR_BLACK);     // black Square
  }

  if (z != _awAccelZ[iAccel]) {
    _awAccelZ[iAccel] = z;
    g_plcd.DisplayNum(bCol, ACCELTEXTY+2, 2, DCLR_RED, z, 3);        // Z

    bAccelVal = z/4;     // Range of 0-31...
    g_plcd.DrawRect(bColP, ACCELGRAPHICY(ACCELTEXTY+2)+1, bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY+2)+10, DCLR_BLUE);          // Blue square
    g_plcd.DrawRect(bColP + bAccelVal, ACCELGRAPHICY(ACCELTEXTY+2)+1, bColP + 64, ACCELGRAPHICY(ACCELTEXTY+2)+10, DCLR_BLACK);     // black Square
  }
}    


//=============================================================================
// DisplaySliders : display information about the sliders - only output if changed
//=============================================================================
void RemoteDisplay::DisplaySliders(byte bLeft, byte bMid, byte bRight) {
  byte bCol;
  byte bJoyVal;

  // First lets output the left slider value and 
  if (_wLSlider != bLeft) {
    _wLSlider = bLeft;
    g_plcd.DisplayNum(0, 6, 2, DCLR_RED, bLeft, 3);        // 
    // Left Slider
    bCol = 24;        // convert to pixel offsets	

    bJoyVal = bLeft/16;     // Range of 0-15...
    g_plcd.DrawRect(bCol, 73, bCol + bJoyVal, 82, DCLR_GREEN);          // Green square
    g_plcd.DrawRect(bCol + bJoyVal, 73, bCol + 16, 82, DCLR_BLACK);     // black Square
  }

  // Now lets process MSlider
  if (_wMSlider != bMid) {
    _wMSlider = bMid;
    g_plcd.DisplayNum(5, 6, 2, DCLR_RED, bMid, 3);      // Y
    bCol = 5 * 8 + 24;        // convert to pixel offsets	
    bJoyVal = bMid/16;     // Range of 0-15...
    g_plcd.DrawRect(bCol, 73, bCol + bJoyVal, 82, DCLR_GREEN);          // Green square
    g_plcd.DrawRect(bCol + bJoyVal, 73, bCol + 16, 82, DCLR_BLACK);     // black Square
  }
  // Now lets process RSlider
  if (_wRSlider != bRight) {
    _wRSlider = bRight;
    g_plcd.DisplayNum(10, 6, 2, DCLR_RED, bRight, 3);      // Y
    bCol = 10 * 8 + 24;        // convert to pixel offsets	
    bJoyVal = bRight/16;     // Range of 0-15...
    g_plcd.DrawRect(bCol, 73, bCol + bJoyVal, 82, DCLR_GREEN);          // Green square
    g_plcd.DrawRect(bCol + bJoyVal, 73, bCol + 16, 82, DCLR_BLACK);     // black Square
  }
}    

//=============================================================================
// DisplayKey : display what key on the keypad is pressed
//=============================================================================
void RemoteDisplay::DisplayKey(char ch) {            // Display a character from the display...
  char abT[2];

  g_plcd.SetFontBackgroundColor(-1);    // Opaque

  abT[0] = ch;
  abT[1] = 0;    // could use the other output
  g_plcd.DisplayString(0, 0, 2, DCLR_WHITE, abT);
}
//=============================================================================
// DisplaySliders : display information about the sliders - only output if changed
//=============================================================================
void RemoteDisplay::DisplayConnectStatus(boolean fConnected) {
  if (_fConnected != fConnected) {
    _fConnected = fConnected;    // remember the new status
    if (fConnected)
      g_plcd.DisplayString(15, 0, 2, DCLR_GREEN, "*");
    else
      g_plcd.DisplayString(15, 0, 2, DCLR_BLACK, " ");
  }
}

//=============================================================================
// DisplayStatus : display status information.  We allow for titles and also
//                 we scroll data between the last 2 lines
//=============================================================================
void RemoteDisplay::DisplayStatus(char *pbTitle, char *pb) {
  byte cb;
  // For this round I am going to display 2 lines of Status.  If no title
  // is specified, I will scroll the previous status line up, else will display
  // Title...
  if (pbTitle) {
    // Draw the text and clear the rest of the line out
    g_plcd.DisplayString(0, 8, 2, DCLR_WHITE, pbTitle);         // draw the text
    cb = strlen(pbTitle);
    g_plcd.DrawRect(cb*8, 8*12, 127, 9*12-1, DCLR_BLACK);    // should clear the rest of the line out.
  } 
  else {
    // Scroll up the stuff from the next line... use screen copy paste command.
    // Need to figure this out...
    g_plcd.ScreenCopyPaste(0, 9*12, 0, 8*12, 128, 12);
  }

  cb = 0;
  if (pb) {
    g_plcd.DisplayString(0, 9, 2, DCLR_WHITE, pb);         // draw the text
    cb = strlen(pb);
  }   
  g_plcd.DrawRect(cb*8, 9*12, 127, 10*12-1, DCLR_BLACK);    // should clear the rest of the line out.

}

//=============================================================================
// DisplayRemoteValue : Display a number that was passed to us...
//=============================================================================
void RemoteDisplay::DisplayRemoteValue(byte iField,  word wVal) {
}

//=============================================================================
// DisplayBatStatus : display information about one of the battery values.
//        if necessary we will draw the actual outline.  Otherwise we will
//        check to see if the value has changed and if so we will update the
//        display...
//=============================================================================
void RemoteDisplay::DisplayBatStatus(byte iBat, byte iPercent) {
  byte bx;
  byte bxFill;

  bx = iBat*38 + 14 + 8;    // Use 38 pixels per battery display save room to display 1 char before...

  if (_abBatPercent[iBat] != iPercent) {
    // This pass will redraw everything...

    // First outline...
    g_plcd.SetSolidPin(false);                                // set to wire frame
    g_plcd.DrawRect(bx, 0, bx + 29, 9, DCLR_WHITE);          // White square
    g_plcd.SetSolidPin(true);
    g_plcd.DrawRect(bx+30, 2, bx + 32, 7, DCLR_WHITE);       // White square

    // Noww lets display the fill the correct percentage of the indicator
    bxFill = bx + 1 + ((word)(7*iPercent))/25;        //(28*iPercent) / 100
    if (bxFill >= (bx + 2))
      g_plcd.DrawRect(bx+1, 1, bxFill, 8, DCLR_GREEN);          // Green square
    if (bxFill < (bx+28)) 
      g_plcd.DrawRect(bxFill+1, 1, bx+28, 8, 0x0);          // black square
  }        


}


//=============================================================================
// ConvertWordToHexString 
//=============================================================================
void ConvertWordToHexString(word w, char *pb) {
  char b;
  byte i;

  pb+=4;
  *pb = 0x0 ;     // make sure null terminated...
  for (i=0; i<4; i++) {
    b = w & 0xf;    // get low nibble
    if (b <= 9)
      *--pb = '0'+b;
    else
      *--pb = 'A'+b-10;
    w >>= 4;
  }    
}



//=============================================================================
// DisplayJoytick : display information about one joystick.
//=============================================================================
void RemoteDisplay::SetBatStatusChar(byte iBat, byte bChar) {    // Set a display char to label the battery symbol (only 1 char)...
  // First pass ignore...
}

//=============================================================================
// DMDisplayCurDL - Choose DL mode - Display current DL
//=============================================================================
void RemoteDisplay::DMDisplayCurDL(word wDL) {
  char ab[5];

  ConvertWordToHexString(wDL, ab);
  g_plcd.DisplayString(9, 7, 2, DCLR_RED, ab);
}

//=============================================================================
// DMDisplayNewDL - Choose DL Mode - Display New DL
//=============================================================================
void  RemoteDisplay::DMDisplayNewDL(word wDL) {
  char ab[5];

  g_plcd.SetFontBackgroundColor(-1);
  ConvertWordToHexString(wDL, ab);
  g_plcd.DisplayString(5, 8, 2, DCLR_RED, ab);
}

//=============================================================================
// DMDisplayNI - display a NodeIdentifier.
//=============================================================================
void RemoteDisplay::DMDisplayNI(char *psz, boolean fCurDL) {
  g_plcd.DisplayString(0, 5, 2, DCLR_BLUE, fCurDL ? "*" : " ");
  g_plcd.DisplayString(1, 5, 2, DCLR_BLUE, psz);
}

//=============================================================================
// DMDisplayStatus - Display status about the operation...
//=============================================================================
void RemoteDisplay::DMDisplayStatus(char *pb) {
  byte cb = 0;
  if (pb) {
    cb = strlen(pb);
    g_plcd.DisplayString(0, 9, 2, DCLR_GREEN, pb);
  }

  g_plcd.DrawRect(cb*8, 9*12, 127, 10*12-1, DCLR_BLACK);    // should clear the rest of the line out.
}
//=============================================================================
// MMDisplayCurMY - Choose MY mode - Display Current My
//=============================================================================
void RemoteDisplay::MMDisplayCurMY(word wMy) {
  char ab[5];
  g_plcd.SetFontBackgroundColor(-1);
  ConvertWordToHexString(wMy, ab);
  g_plcd.DisplayString(9, 7, 2, DCLR_RED, ab);
}

//=============================================================================
// MMDisplayNewMY - Choose My mode - Display New My
//=============================================================================
void RemoteDisplay::MMDisplayNewMY(word wMy) {
  char ab[5];

  g_plcd.SetFontBackgroundColor(-1);
  ConvertWordToHexString(wMy, ab);
  g_plcd.DisplayString(5, 8, 2, DCLR_RED, ab);
}


//=============================================================================
// MMDisplayStatus - Display status about the operation...
//=============================================================================
void RemoteDisplay::MMDisplayStatus(char *pb) {
  g_plcd.DisplayString(0, 9, 2, DCLR_GREEN, pb);
}



//=============================================================================
// ReadTouchScreen: 
//=============================================================================
word  RemoteDisplay::ProcessTouch(void) {

  word wTouchState, wXTouch, wYTouch;
  word   wRet;
  byte iBtnPressed;

  // we will check to see if the user is touching the display.  If we get a down event, we will see if the touch 
  // location is on one of our buttons. If so we draw the button in a pressed state, and wait for the button to come up. 
  // if any moves during this time, we see if we are still within the same button, if not, unpress it, then see if on
  // other button and show pressed... If release is on the button, we will show it come up and return the button
  // that was pressed to the caller, else return 0xff..

  // Test of touchpad...
  if(!g_plcd.FTouchEventsAvail())
    return 0xffff;

  g_plcd.GetTouchPoint(&wXTouch, &wYTouch);

  if (g_fDebugOutput) {
    DBGSerial.print("Touch: ");
    DBGSerial.print(wXTouch, DEC);
    DBGSerial.print(" ");
    DBGSerial.println(wYTouch, DEC);
  }
  for (iBtnPressed=0; iBtnPressed < NUM_BUTTONS; iBtnPressed++) {
    if ((wRet = g_abtns[iBtnPressed].ProcessTouch(wXTouch, wYTouch)))
      break;    // if it return non-zero then it was processed by that object
  }
  if (!wRet)
    wRet = g_asldrs[0].ProcessTouch(wXTouch, wYTouch);
  if (!wRet)
    wRet = g_asldrs[1].ProcessTouch(wXTouch, wYTouch);



  if (!wRet)
    return (0xffff);

  return wRet;
}

//=============================================================================
// GetSliderValue: 
//=============================================================================
byte  RemoteDisplay::GetSliderValue(byte i) {
  return g_asldrs[i].GetValue();
}


