//=============================================================================================================
// Extract out the Picaso SGC into a class that can be reused for other projects...
//=============================================================================================================

#ifndef _OLEDSGC_H_
#define _OLEDSGC_H_

class DISPOBJ;

class OLEDSGC {
public:
  // The Init function.
  void Init(byte bResetPin);    // Pass in an optional Init reset pin that is used to 

  // Text functions
  void SetFontBackgroundColor(unsigned long   ulClr) {_ulFontBackgroundColor = ulClr;};
  void DisplayString(byte x, byte y, byte bFont, word wColor, const char *pb);
  void DisplayGraphicString(word x, word y, byte bFont, word wColor, const char *pb);
  void DisplayNum(byte x, byte y, byte bFont, word wColor, word w, byte bCnt);

  // Graphic functions
  void ClearScreen(void);
  void SetSolidPin(boolean fSolid) {_fSolid = fSolid;};
  void DrawRect(word x1, word y1, word x2, word y2, word wColor);
  void ScreenCopyPaste(word xS, word yS, word xD, word yD, word dx, word dy);

  // Touch functions
//  boolean GetTouchStatus(byte bMode, word *wX, word *wY);    // get the two words of data associated with the touch panel
  boolean FTouchEventsAvail();
  boolean GetTouchPoint(word *pwX, word *pwY);
  boolean FTouched();
  boolean ProcessTouch(void);


private:    
  boolean _fCommandActive;
  DISPOBJ *_pdoFirst;        // pointer to first display object
  unsigned long _ulFontBackgroundColor = -1;  // -1 says transparant
  boolean _fSolid = false;

};  

// Declare one global instance of this class.
extern OLEDSGC g_plcd;

class DISPOBJ {
public:
  inline void    Enable(boolean fEnabled);

  virtual void Draw(void)=0;
  virtual word ProcessTouch(word x, word y)=0;
  friend class OLEDSGC;
protected:
  DISPOBJ *pdoNext;        // Next display object
  word    _x;             //TL
  word    _y;    
  word    _w;            
  word    _h;
boolean _fEnabled : 
  1;
boolean _fPressed : 
  1;
boolean _fTouchActive : 
  1;
};

class BUTTON  : 
public DISPOBJ {
public:
  void Init (word x, word y, word dx, word dy, word wBtnClr, word wHiClr, word wTextClr, char *psz, word wVal);
  virtual void Draw(void);
  virtual word ProcessTouch(word x, word y);

  // some button specific functions
  inline void SetPressed(boolean fPressed)  {
    _fPressed = fPressed;
  };        
private:    
  word _wBtnClr;
  word _wHiClr;
  word _wTextClr;
  char *_psz;
  word  _wVal;

  //        boolean _fPressed;
};

class SLIDER  : 
public DISPOBJ {
public:
  void Init (word x, word y, word dx, word dy, word wClr, word wHiClr, word wBackClr, word wMin, word wMax, word wVal);
  virtual void Draw(void);
  virtual word ProcessTouch(word x, word y);

  // some button specific functions
  inline word GetValue(void) {
    return _wVal;
  };
  void SetValue(word wVal);

  inline void SetPressed(boolean fPressed)  {
    _fPressed = fPressed;
  };        
private:    
  inline void MapValToDispVal(void);
  inline word MapDispValToVal(word wXorY);

  word _wClr;
  word _wHiClr;
  word _wBackClr;
  boolean _fPressed;
  boolean _fVert;        // is the slider vertical or horizontal?

  word _wMin;
  word _wMax;
  word _wVal;

  word _wDispVal;        // This is the split X or Y value of what the current value maps to..
};



#endif //_OLEDSGC_H_

