
//=============================================================================
//Project DIY Remote
//Software version: V1.0
//Date: July 2011
//Programmer:   Kurt Eckhardt(KurtE) converted to C and Arduino
//=============================================================================
//
// Seeeduino Mega Pro...
//
//#define DEBUG_SAVED_LIST
//

//KNOWN BUGS:
//    - Lots ;)
//
//=============================================================================
// Header Files
//=============================================================================
#include <Adafruit_GFX.h>    // Core graphics library
#include <SPI.h>
#include <Wire.h>      // this is needed even tho we aren't using it
#include <Adafruit_ILI9341.h>
#include <Adafruit_STMPE610.h>

#include <EEPROM.h>
#include <Streaming.h>
#include "globals.h"
#include <pins_arduino.h>

//=============================================================================
// Tables
//=============================================================================
static const word c_KeypadMapping[] = {
    /*    0x1, 0x2, 0x3, 0xA, 
     0x4, 0x5, 0x6, 0xB, 
     0x7, 0x8, 0x9, 0xC, 
     0x0, 0xF, 0xE, 0xD}; */
    1, 4, 7, 0,
    2, 5, 8, 0xf,
    3, 6, 9, 0xe,
    0xa, 0xb, 0xc, 0xd};

//=============================================================================
// Defines
//=============================================================================


#define FIRST_KEYPAD_IOPIN    22
#define SOUND_PIN    6

enum {
    AP_RJOYLR=0, AP_RJOYUD, AP_LJOYLR, AP_LJOYUD, AP_RSLIDER, AP_LSLIDER, AP_RPOT, AP_LPOT,  AP_MSLIDER};

// Define command buttons.
#define CMDB_NEXT    0x4    // Above
#define CMDB_PREV    0x8    // Below
#define CMDB_ENTR    0x2    // left
#define CMDB_CNCL    0x1    // right
// Some battery defines
#define BATTERY_PIN    15    // Analog pin 15
#define BATTERY_MIN    256    // Min Analog value to use to report 0 256 is about 5V
#define BATTERY_MAX    384    // Max analog value to report 100% about 7.5 v  = (V * 1024) / 20

//=============================================================================
// Global Variables
//=============================================================================
word    wKeypadPrev;

// Mode information
byte              g_bMode;                                       // What mode are we in...
boolean           g_fNewMode;                                    // Have we just entered this mode?

// Analog information
word              g_aawRawAnalog[8][NUMANALOGS];                 // Keep 8 raw values for each analog input
word              g_awRawBatAnalog[8];
word              g_awRawSums[NUMANALOGS];                       // sum of the raw values.
word              g_wRawBatterySum;
byte              g_bCmdButtons;                                // which command buttons are pressed
byte              g_bCmdButtonsPrev;                            // which were set before...

    word wButtons;

static const boolean  g_afAnalogUseMids[] = {
    1,1,1,1,0,0,1,1, 0};       // Do we use the mid points when we normalize
static const word g_awAnalogMins[] = {
    0*8,0*8,0*8,0*8,0*8,0*8, 0*8, 0*8, 0*8};  // Will have a calibrate later where we read these in (if necessary)
static const word g_awAnalogMaxs[] = {
    1023*8,1023*8,1023*8,1023*8,1023*8,1023*8, 1023*8, 1023*8, 1023*8}; // dito

word              g_awAnalogMids[NUMANALOGS];                // This is our observed center points at init time. 
byte        g_iRaw;                           // which raw value are we going to read into

word        g_aw0Prev;
boolean     g_fShowDebugPrompt;
boolean     g_fDebugOutput;
boolean     g_fVerbose;                // Do we want verbose output?
byte        g_bDispLoopCntr;                // Use this to decide which fields we should display per loop

RemoteDisplay g_display;                // Define our display here.


// Globals used in some of the config code.
byte            g_cNDList;                    // count of items in the list is saved ND list 
byte            g_iNDList;                   // Index to current one
word            g_wNewVal;                    // The new value
unsigned long   g_ulTimeLastMsg;             // Time of last message;
// forward definitions.
extern int TermReadln(byte *psz, byte cbMax);

//--------------------------------------------------------------------------
// SETUP: the main arduino setup function.
//--------------------------------------------------------------------------
void setup(){
    delay(2000);  // give time to setup
    DBGSerial.begin(57600);
    MSound(4, 100, 440, 100, 494, 100, 523, 100, 588);
    DBGSerial.write("Program Start\n\r");
    g_fDebugOutput = false;			// start with it off!
    g_fVerbose = false;
    g_fShowDebugPrompt = true;

    DBGSerial.println("Before Display Init");    
    g_display.Init();
    DBGSerial.println("After Display Init");    
    g_bMode = MODE_NORMAL;
    g_display.SwitchToDisplayMode(MODE_NORMAL);

}

//=============================================================================
// Loop: the main arduino main Loop function
//=============================================================================
void loop(void)
{
    char ch;
    word w;
    
    // We also have a simple debug monitor that allows us to 
    // check things. call it here..
    if (TerminalMonitor())
        return;           
    // Note we may need to add additional support if we add in other controls like sliders...
    w = g_display.ProcessTouch();
    
    if (g_fDebugOutput) {
        if (w && (w != 0xffff)) {
            DBGSerial.print("TV: ");
            DBGSerial.println(w, HEX);
        }
    }
    
    wButtons = 0;
    g_bCmdButtons = 0;        // get the command value
    if ((w & 0xff00) == 0x100)  {
        wButtons = 1 << (w & 0xff);    // Get the byte value...
        if (g_fDebugOutput) {
            DBGSerial.print("B: ");
            DBGSerial.println(wButtons, HEX);
        }
    }
    
    else if ((w & 0xff00) == 0x200)
        g_bCmdButtons = (w & 0xff);        // get the command value
    
      if ( g_bCmdButtons & CMDB_NEXT) {
            // Next button pressed.
            g_bMode++;
            if (g_bMode >= MODE_MAX)        // Handle wrap around...
                g_bMode = MODE_NORMAL;
            g_display.SwitchToDisplayMode(g_bMode);
            g_ulTimeLastMsg = (unsigned long)-1;
            g_fNewMode = true;
        } 
        else if ( g_bCmdButtons & CMDB_PREV) {
            if (g_bMode)
                g_bMode--;        // decrement
            else
                g_bMode = MODE_MAX-1;    // wrap back    
            g_display.SwitchToDisplayMode(g_bMode);
            g_fNewMode = true;
            g_ulTimeLastMsg = (unsigned long)-1;
        }

    switch (g_bMode) {
    case MODE_NORMAL:
        // Now lets call off to maybe output packet...
        break;
    case MODE_CHANGE_DEST_LIST:
        ChangeDestMode();
        break;

    case MODE_CHANGE_MY:
        ChangeMyMode();
        break;

    case MODE_CALIBRATE:
        CalibrateMode();
        break;    
    } // End of our mode case statement...

}

//==============================================================================
// ChangeMyMode - Handles display and UI when we are in the change My
//==============================================================================
void ChangeMyMode(void) {
    word w;
    byte i;
    if (g_fNewMode) {
        g_fNewMode = false;

        // Display our initial informatation for this mode.
#if 0
        g_display.MMDisplayCurMY(g_diystate.wXBeeMY);
        g_wNewVal = g_diystate.wXBeeMY;
        g_display.MMDisplayNewMY(g_wNewVal);
#endif
    }

    // See if one of the keypad buttons is pressed.     
    if (wButtons && (wButtons != wKeypadPrev)) {  // should probably handle cases of multiple buttons...
        // need to convert the bit number to hex value...
        w = wButtons;
        for (i=0; !(w & 1) && (i<16); i++) {
            w = w >> 1;
        }

        g_wNewVal = (g_wNewVal << 4) | i;
        g_display.MMDisplayNewMY(g_wNewVal);
    }        

    // See if the user clicks on the OK button, to apply the new value.
    if ((g_bCmdButtons & CMDB_ENTR) && (g_bCmdButtons != g_bCmdButtonsPrev)) {

#if 0
      g_diystate.wXBeeMY = g_wNewVal;
        APISetXBeeHexVal('M', 'Y', g_diystate.wXBeeMY);

        // and update saved XBee info
        UpdateSavedXBeeInfo();

        g_display.MMDisplayCurMY(g_diystate.wXBeeMY);
#endif
    }    

}

//==============================================================================
// CDM_DisplayNIItem - Display an NDList Item
//==============================================================================
void CDM_DisplayNIItem() {
    // Lets read in the text for this node
    char ab[CBNIMAX+1];
    word w;
    byte j;
#if 0
    // First lets get the My(DL) for this item...
    g_wNewVal = (EEPROM.read(XBNDDM_AMY + g_iNDList*2) << 8) + EEPROM.read(XBNDDM_AMY+g_iNDList*2+1);
    g_display.DMDisplayNewDL(g_wNewVal);

    // Then lets get the Node Identifier text for this item...
    w =  g_iNDList * CBNIMAX;
    for (j=0; j < CBNIMAX; j++) {
        ab[j] = EEPROM.read(XBNDDM_ANDI+w+j);
    }
    ab[CBNIMAX]=0;
    g_display.DMDisplayNI(ab, g_diystate.wXBeeDL == g_wNewVal);
#endif
}

//==============================================================================
// ChangeDestMode - Handles display and UI when we are in the change destination
//         mode, 
//==============================================================================
void ChangeDestMode(void) {
    word w;
    byte i;
    boolean fCurDL;
#if 0    
    if (g_fNewMode) {
        // Display our initial informatation for this mode.

        g_display.DMDisplayCurDL(g_diystate.wXBeeDL);
        g_wNewVal = g_diystate.wXBeeDL;
        g_display.DMDisplayNewDL(g_wNewVal);
        g_fNewMode = false;

        // Lets see if our current DL is in our saved list, if so lets get the index...
        g_cNDList = EEPROM.read(XBeeNDDMCache);
        if (g_cNDList> CMAXNDLIST)
            g_cNDList = 0;

        for (g_iNDList = 0; g_iNDList < g_cNDList; g_iNDList++) {
            w = (EEPROM.read(XBNDDM_AMY + g_iNDList*2) << 8) + EEPROM.read(XBNDDM_AMY+g_iNDList*2+1);
            if (fCurDL = (w == g_wNewVal))
                break;
        }

        if ( g_iNDList == g_cNDList)
            g_iNDList = 0;    // did not find... display first one (if any)

        // Display information about this item...    
        if (g_cNDList) {
            CDM_DisplayNIItem();
        }            
    }

    // Lets see if the user wants us to do a scan for new XBee Devices...
    if ((diyp.s.bButtons2 & 1) && !(diypSave.s.bButtons2 & 1)) {
        g_display.DMDisplayStatus("Start Scan");
        UpdateNDListFromXBee();        // Ok lets see if we can find any new XBees out there...
        g_display.DMDisplayStatus("Completed");
        g_fNewMode = true;    // setup to reinitialize things.
    }


    // Handle scrolling through list with right joystick up/down
    if (g_cNDList) {
        // Only process if the last time was > some time threshold...
        // Hack only do this once per movement, could/should use timer...
#ifdef DEBUG        
        if (g_fDebugOutput) {
            if (((diyp.s.bRJoyUD < 64) && (diypSave.s.bRJoyUD >= 64)) ||
                ((diyp.s.bRJoyUD > 196) && (diypSave.s.bRJoyUD <= 196))) {

                DBGSerial.print(diyp.s.bRJoyUD, DEC);
                DBGSerial.print(":");
                DBGSerial.print(diypSave.s.bRJoyUD, DEC);
                DBGSerial.print(":");
                DBGSerial.print(g_iNDList, DEC);
                DBGSerial.print(":");
                DBGSerial.println(g_cNDList, DEC);
            }
        }        
#endif

        if ((diyp.s.bRJoyUD < 64) && (diypSave.s.bRJoyUD >= 64) && (g_iNDList > 0)) {
            g_iNDList--;
            CDM_DisplayNIItem();

        } 
        else if ((diyp.s.bRJoyUD > 196) && (diypSave.s.bRJoyUD <= 196) && (g_iNDList < (g_cNDList-1))) {
            g_iNDList++;
            CDM_DisplayNIItem();
        }
    }


    // See if one of the keypad buttons is pressed.     
    if (wButtons && (wButtons != wKeypadPrev)) {  // should probably handle cases of multiple buttons...
        // need to convert the bit number to hex value...
        w = wButtons;
        for (i=0; !(w & 1) && (i<16); i++) {
            w = w >> 1;
        }

        g_wNewVal = (g_wNewVal << 4) | i;
        g_display.DMDisplayNewDL(g_wNewVal);
    }        

    // See if the user clicks on the OK button, to apply the new value.
    if ((g_bCmdButtons & CMDB_ENTR) && (g_bCmdButtons != g_bCmdButtonsPrev)) {

        g_diystate.wXBeeDL = g_wNewVal;
        g_diystate.wAPIDL = g_wNewVal;
        SetXBeeDL(g_diystate.wXBeeDL);

        // and update saved XBee info
        UpdateSavedXBeeInfo();

        g_display.DMDisplayCurDL(g_diystate.wXBeeDL);
    }    
#endif
}


//==============================================================================
// CalibrateMode: Go through and Calibrate all of the joysticks from their minimum
//     to maximum values to allow us to better output a standardized range...
//==============================================================================
void CalibrateMode(void) {
}


//==============================================================================
// TerminalMonitor - Simple background task checks to see if the user is asking
//    us to do anything, like update debug levels ore the like.
//==============================================================================
boolean TerminalMonitor(void)
{
    byte szCmdLine[20];
    int ich;
    // See if we need to output a prompt.
    if (g_fShowDebugPrompt) {
        Serial.println("Arduino DIY Remote Monitor");
        Serial.println("D - Toggle debug on or off");
        Serial.println("V - Choose Verbose on or Off");
        Serial.println("R - Choose Remote Dest");
        Serial.println("Mxxxx - Set our address in hex");
        g_fShowDebugPrompt = false;
    }

    // First check to see if there is any characters to process.
    while (Serial.peek() == 10)    
        Serial.read();        // strip off any leading LF characters...

    if (Serial.available()) {
        ich = TermReadln(szCmdLine, sizeof(szCmdLine));
        Serial.print("Serial Cmd Line:");        
        Serial.write(szCmdLine, ich);
        Serial.println("!!!");

        // So see what are command is.
        if (ich == 0) {
            g_fShowDebugPrompt = true;
        } 
        else if ((ich == 1) && ((szCmdLine[0] == 'd') || (szCmdLine[0] == 'D'))) {
            g_fDebugOutput = !g_fDebugOutput;
            if (g_fDebugOutput) 
                Serial.println("Debug is on");
            else
                Serial.println("Debug is off");
        } 
        else if ((ich == 1) && ((szCmdLine[0] == 'v') || (szCmdLine[0] == 'V'))) {
            g_fVerbose = !g_fVerbose;
            if (g_fVerbose) 
                Serial.println("Verbose is on");
            else
                Serial.println("Verbose is off");
        } 

        return true;
    }
    return false;
} 


//==============================================================================
// TermReadln - Simple helper function to read in a command line from the termainal
//==============================================================================
int TermReadln(byte *psz, byte cbMax) 
{
    byte ich;
    int ch;
    // For now assume we receive a packet of data from serial monitor, as the user has
    // to click the send button...
    ich = 0;
    while (((ch = Serial.read()) != 13) && (ich < cbMax)) {
        if ((ch != -1) && (ch != 10)) {
            *psz++ = ch;
            ich++;
        }
    }
    *psz = '\0';    // go ahead and null terminate it...
    return ich;
}    



//=============================================================================
// InitKeypad: Lets setup the IO pins associated with the keypad
//=============================================================================

void InitKeypad(void) {
#if 0
    // Assume first four pins are the rows and the next four pins are the cols.
    byte bPin;

    // Init Rows
    for (bPin = FIRST_KEYPAD_IOPIN; bPin < FIRST_KEYPAD_IOPIN+4; bPin++) {
        pinMode(bPin, INPUT);        // set all of the pins to inptu
    }

    // Init Cols
    for (; bPin < FIRST_KEYPAD_IOPIN+8; bPin++) {
        digitalWrite(bPin, HIGH);    // set the in high which when input, enables PU
        pinMode(bPin, INPUT);        // set all of the pins to input
    }
#endif    
}        





//=============================================================================
// ReadKeypad: Now lets read the keypad and see if any buttons are pressed
//=============================================================================
word ReadKeypad(char *pch) {
#if 0    
    word wRet = 0;
    byte bRow;
    byte bCol;
    byte bBit;

    *pch = ' ';    // assume a blank
    for (bRow = FIRST_KEYPAD_IOPIN; bRow < FIRST_KEYPAD_IOPIN+4; bRow++) {
        // Set the row to low
        pinMode(bRow, OUTPUT);
        digitalWrite(bRow, LOW);

        for (bCol = FIRST_KEYPAD_IOPIN+4; bCol <  FIRST_KEYPAD_IOPIN+8; bCol++) {
            if (digitalRead(bCol) == LOW) {
                bBit =  c_KeypadMapping[(bRow-FIRST_KEYPAD_IOPIN)*4 + bCol-(FIRST_KEYPAD_IOPIN+4)];
                wRet |= 1 << bBit;
                if (bBit <= 9)
                    *pch = '0'+ bBit;
                else
                    *pch = 'A'+ bBit-10;
            }
        }
        // restore the IO pin to be an input...
        pinMode(bRow, INPUT);    // 
    }

    return wRet;
#else
    return 0;
#endif    
}



//==============================================================================
// ParseHexString
//==============================================================================
word ParseHexString(byte *psz) {
    word w = 0;
    while (*psz) {
        if (*psz >= '0' && *psz <= '9')
            w = w*16 + *psz-'0';
        else if (*psz >= 'a' && *psz <= 'f')
            w = w*16 + *psz-'a' + 10;
        else if (*psz >= 'A' && *psz <= 'F')
            w = w*16 + *psz-'A' + 0;
        psz++;
    }
    return w;
}


// BUGBUG:: Move to some library...
//==============================================================================
//    SoundNoTimer - Quick and dirty tone function to try to output a frequency
//            to a speaker for some simple sounds.
//==============================================================================
void SoundNoTimer(uint8_t _pin, unsigned long duration,  unsigned int frequency)
{
    volatile uint8_t *pin_port;
    volatile uint8_t pin_mask;
    long toggle_count = 0;
    long lusDelayPerHalfCycle;

    // Set the pinMode as OUTPUT
    pinMode(_pin, OUTPUT);

    pin_port = portOutputRegister(digitalPinToPort(_pin));
    pin_mask = digitalPinToBitMask(_pin);

    toggle_count = 2 * frequency * duration / 1000;
    lusDelayPerHalfCycle = 1000000L/(frequency * 2);

    // if we are using an 8 bit timer, scan through prescalars to find the best fit
    while (toggle_count--) {
        // toggle the pin
        *pin_port ^= pin_mask;

        // delay a half cycle
        delayMicroseconds(lusDelayPerHalfCycle);
    }    
    *pin_port &= ~(pin_mask);  // keep pin low after stop

}

void MSound(byte cNotes, ...)
{
    va_list ap;
    unsigned int uDur;
    unsigned int uFreq;
    va_start(ap, cNotes);

    while (cNotes > 0) {
        uDur = va_arg(ap, unsigned int);
        uFreq = va_arg(ap, unsigned int);
#ifdef SOUND_PIN
        SoundNoTimer(SOUND_PIN, uDur, uFreq);
#endif        
        cNotes--;
    }
    va_end(ap);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////


