//=============================================================================
// Display.h XBee Support for the DIY Remote control
// This class is used to hopefully allow multiple implentations, depending on
// which type of display we are using.  First version will be for the OLED...
//=============================================================================
#ifndef _DISPLAY_H_
#define _DISPLAY_H_
#if ARDUINO>99
#include <Arduino.h>
#else
#include <WProgram.h> 
#endif
#include <Adafruit_ILI9341.h>

// Also lets define some colors here to make it easier to read some of the code.
#define DCLR_RGB(red8,green8,blue8)  tft.color565(red8,green8,blue8)
#define DCLR_WHITE        ILI9341_WHITE
#define DCLR_BLACK        ILI9341_BLACK
#define DCLR_RED          ILI9341_RED
#define DCLR_BLUE         ILI9341_BLUE
#define DCLR_GREEN        ILI9341_GREEN

class RemoteDisplay {
public:
  void     Init(void);               // do whatever is needed for initial init

  void    SwitchToDisplayMode(byte bMode);    // Init to specified mode 0=default

  void    StartDispalyUpdate(void);  // Lets display know we are about to change things...
  void    DisplayKeypad(char ch);    // display the current keypad data.
  void    DisplayJoystick(byte iJoy, byte x, byte y);    // only 2 on this joystick...
  void    DisplayAccels(byte iJoy, byte x, byte y, byte z);
  void    DisplaySliders(byte bLeft, byte bMid, byte bRight);
  void    DisplayKey(char ch);            // Display a character from the display...
  void    DisplayConnectStatus(boolean fConnected);

  void    DisplayRemoteValue(byte iField,  word wVal);
  void    DisplayStatus(char *pbTitle, char *pb);
  void    DisplayBatStatus(byte iBat, byte iPercent);    
  void    SetBatStatusChar(byte iBat, byte bChar);    // Set a display char to label the battery symbol (only 1 char)...

  // Functions for Change DL mode
  void    DMDisplayCurDL(word wDL);
  void    DMDisplayNewDL(word wDL);
  void    DMDisplayNI(char *psz, boolean fCurDL);
  void    DMDisplayStatus(char *pb);

  // Functions for Change MY mode
  void    MMDisplayCurMY(word wMy);
  void    MMDisplayNewMY(word wMy);
  void    MMDisplayStatus(char *pb);

  // Lets try some Touch screen stuff...
  word ProcessTouch(void);                                   // May need additional information...
  byte GetSliderValue(byte i);                                // 0= left, 1 = right

private:
  word _awJoyX[2];                //  Joystick values
  word _awJoyY[2];                // Could be bytes b
  word _awAccelX[];
  word _awAccelY[];
  word _awAccelZ[];
  word _awJoyZ[2];                
  word _wRSlider;                // Likewise for sliders
  word _wMSlider;                
  word _wLSlider;
  byte _abBatPercent[3];        // We support up to 3 batteries (0=Ours, 1 and 2 is for robot VS and VL possible...)
  boolean _fConnected;

};

extern RemoteDisplay g_display;
#endif


